//+------------------------------------------------------------------+
//|                                       SmallCatZzFractal-v1.1.mq4 |
//|                                  Based on : Custom ZZ & Fractals |
//|                                         by SmallCat@ForexFactory |
//| v-1.1                                                            |
//| Need other indicators : Fractals & Killian's Custom ZZ           |
//+------------------------------------------------------------------+

#property copyright "SmallCat"
#property link      "SmallCat@ForexFactory"

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 White
#property indicator_color2 Magenta
#property indicator_color3 DeepSkyBlue
#property indicator_color4 Gold
#property indicator_width1 0
#property indicator_width2 0
#property indicator_width3 1
#property indicator_width4 1
//---
string          version        = "1.1";

//---- input parameters
extern int      ExtDepth       = 13;
extern int      ExtDeviation   = 8;
extern int      ExtBackstep    = 5;
extern double   distance       = 0.00018;      

//---- buffers
double UpperFractal[];
double LowerFractal[];
double UpperZigzag[];
double LowerZigzag[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()  {
   IndicatorBuffers(4);
   IndicatorShortName("SmallCatZzFractal-v" + version + " ( " + ExtDepth + "," + ExtDeviation +  "," + ExtBackstep + ") ");
   SetIndexBuffer(0,UpperFractal);
   SetIndexBuffer(1,LowerFractal);  
   SetIndexBuffer(2,UpperZigzag);
   SetIndexBuffer(3,LowerZigzag);
   SetIndexStyle(0,DRAW_ARROW,STYLE_SOLID,indicator_width1,indicator_color1);
   SetIndexArrow(0,230);
   SetIndexStyle(1,DRAW_ARROW,STYLE_SOLID,indicator_width2,indicator_color2);
   SetIndexArrow(1,228);
   SetIndexStyle(2,DRAW_ARROW,STYLE_SOLID,indicator_width3,indicator_color3);
   SetIndexArrow(2,253);
   SetIndexStyle(3,DRAW_ARROW,STYLE_SOLID,indicator_width3,indicator_color4);
   SetIndexArrow(3,253);
   SetIndexEmptyValue(0,0.0);
   SetIndexEmptyValue(1,0.0);
   SetIndexEmptyValue(2,0.0);
   SetIndexEmptyValue(3,0.0);
   SetIndexLabel(0,"UpperFractal");
   SetIndexLabel(1,"LowerFractal");
   SetIndexLabel(2,"UpperZigzag");
   SetIndexLabel(3,"LowerZigzag");
   return(0);
}

//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit() {
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start() {
   int limit, i, tf = Period();
   string cur = Symbol();
   int counted_bars = IndicatorCounted(); 
   if (counted_bars<0) return(-1);
   if (counted_bars>0) counted_bars--;
   limit = Bars - counted_bars; 
   //---
   double dHigh,dLow,dHighPrev,dLowPrev,dHighPrev2,dLowPrev2;
   double upper_fractal,lower_fractal,upper_fractalPrev,lower_fractalPrev,upper_fractalPrev2,lower_fractalPrev2;
   double zigzagprice,prevbarZigzagprice,prevbar2Zigzagprice;
   //--
   for (i=limit; i>=0; i--)  {
      dHigh                = iHigh(cur,tf,i);
      dLow                 = iLow(cur,tf,i);
      dHighPrev            = iHigh(cur,tf,i+1);
      dLowPrev             = iLow(cur,tf,i+1);
      dHighPrev2           = iHigh(cur,tf,i+2);
      dLowPrev2            = iLow(cur,tf,i+2);
      upper_fractal        = iFractals (cur, tf, MODE_UPPER, i);
      lower_fractal        = iFractals (cur, tf, MODE_LOWER, i);
      upper_fractalPrev    = iFractals (cur, tf, MODE_UPPER, i+1);
      lower_fractalPrev    = iFractals (cur, tf, MODE_LOWER, i+1);
      upper_fractalPrev2   = iFractals (cur, tf, MODE_UPPER, i+2);
      lower_fractalPrev2   = iFractals (cur, tf, MODE_LOWER, i+2);
      zigzagprice          = iCustom(cur,tf,"Custom ZigZag",ExtDepth,ExtDeviation,ExtBackstep,0,i);  
      prevbarZigzagprice   = iCustom(cur,tf,"Custom ZigZag",ExtDepth,ExtDeviation,ExtBackstep,0,i+1); 
      prevbar2Zigzagprice  = iCustom(cur,tf,"Custom ZigZag",ExtDepth,ExtDeviation,ExtBackstep,0,i+2);
      //---
      if (prevbar2Zigzagprice != 0) {
         if (prevbar2Zigzagprice == dHighPrev2) { // upperZZ
            UpperZigzag[i+2] = dHighPrev2;  
         }
         if (prevbar2Zigzagprice == dLowPrev2) {
            LowerZigzag[i+2] = dLowPrev2;
         }      
      }
      //---
      if (upper_fractalPrev2 != 0) {
         UpperFractal[i+2] = dHighPrev2 + distance;
      }
      if (lower_fractalPrev2 != 0) {
         LowerFractal[i+2] = dLowPrev2 - distance;
      }
   } // ende for
   //---  
   return(0);
} // ende start
//+------------------------------------------------------------------+